/****************************************************************************/
/*!
\par Project:  Video For DreamCast
\file          mpgplayer.c
\author        La Cible (la.cible@free.fr)
\note          This source code file isn't copyright.
*/
/****************************************************************************/
/**
 * mpeg2 player
 */
#include <kos.h>
#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>

#include "video_out.h"
#include "mpeg2.h"
#include "mm_accel.h"
#include  <vfdc.h>

#define BUFFER_SIZE 4096
static uint8_t buffer[BUFFER_SIZE];
static FILE * in_file;
static mpeg2dec_t mpeg2dec;
static char stop=0;

static void ps_loop (int (*callback)())
{
    static int mpeg1_skip_table[16] = {
	     1, 0xffff,      5,     10, 0xffff, 0xffff, 0xffff, 0xffff,
	0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff
    };

    uint8_t * buf;
    uint8_t * end;
    uint8_t * tmp1;
    uint8_t * tmp2;
    int complain_loudly;
    int PacketSize;
    int Skipping;



    complain_loudly = 1;
    buf = buffer;
	
    do {

		if (callback) 
		{
			int ret = (*callback)();
			if (ret != VF4DC_OK)
			{	
				stop=1;
				break;
			}
			
		}
		
	
	end = buf + fread (buf, 1, buffer + BUFFER_SIZE - buf, in_file);
	buf = buffer;


	while (buf + 4 <= end) {
	    /* check start code */
	    if (buf[0] || buf[1] || (buf[2] != 0x01)) {
		if (complain_loudly) {
		    printf ("missing start code at %#lx\n",
			     ftell (in_file) - (end - buf));
		    if ((buf[0] == 0) && (buf[1] == 0) && (buf[2] == 0))
			printf ("this stream appears to use "
				 "zero-byte padding before start codes,\n"
				 "which is not correct according to the "
				 "mpeg system standard.\n"
				 "mp1e was one encoder known to do this "
				 "before version 1.8.0.\n");
		    complain_loudly = 0;
		}
		buf++;
		continue;
	    }

	    switch (buf[3]) {
	    case 0xb9:	/* program end code */
			//printf ("byebye !");
			
			return;
	    case 0xba:	/* pack header */
		/* skip */
		if ((buf[4] & 0xc0) == 0x40)	/* mpeg2 */
		    tmp1 = buf + 14 + (buf[13] & 7);
		else if ((buf[4] & 0xf0) == 0x20)	/* mpeg1 */
		    tmp1 = buf + 12;
		else if (buf + 5 > end)
		    goto copy;
		else {
		    printf ("weird pack header\n");
		    return;
		}
		if (tmp1 > end)
		    goto copy;
		buf = tmp1;
		break;
            case 0xc0:  /* audio */
                PacketSize=(buf[4] << 8) + buf[5];
                Skipping=buf[8]+3;
                PacketSize-=Skipping;
               // AudioFifoIn (&buf[6+Skipping],PacketSize);
                tmp1 = buf + 6 + (buf[4] << 8) + buf[5];
                if (tmp1 > end)
                    goto copy;
                buf = tmp1;
                break;
	    case 0xe0:	/* video */
		tmp2 = buf + 6 + (buf[4] << 8) + buf[5];
		if (tmp2 > end)
		    goto copy;
		if ((buf[6] & 0xc0) == 0x80)	/* mpeg2 */
		    tmp1 = buf + 9 + buf[8];
		else {	/* mpeg1 */
		    for (tmp1 = buf + 6; *tmp1 == 0xff; tmp1++)
			if (tmp1 == buf + 6 + 16) {
			    printf ("too much stuffing\n");
			    buf = tmp2;
			    break;
			}
		    if ((*tmp1 & 0xc0) == 0x40)
			tmp1 += 2;
		    tmp1 += mpeg1_skip_table [*tmp1 >> 4];
		}
		if (tmp1 < tmp2)
			mpeg2_decode_data (&mpeg2dec, tmp1, tmp2);
		buf = tmp2;
		break;
	    default:
		if (buf[3] < 0xb9) {
		    printf ("looks like a video stream, not system stream\n");
		    return;
		}
		/* skip */
		tmp1 = buf + 6 + (buf[4] << 8) + buf[5];
		if (tmp1 > end)
		    goto copy;
		buf = tmp1;
		break;
	    }
	}

	if (buf < end) {
	copy:
	    /* we only pass here for mpeg1 ps streams */
	    memmove (buffer, buf, end - buf);
	}
	buf = buffer + (end - buf);


    } while (end == buffer + BUFFER_SIZE);
}

/*
static void es_loop (void)
{
    uint8_t * end;
	cont_cond_t cond;
		
    do {
	cont_get_cond(maple_first_controller(), &cond);
	if(!(cond.buttons & CONT_START))
	{	
		stop=1;
	    break;
	}
	end = buffer + fread (buffer, 1, BUFFER_SIZE, in_file);
	mpeg2_decode_data (&mpeg2dec, buffer, end);
    } while((end == buffer + BUFFER_SIZE) );
 
}
*/
int play_mpeg2(char *filename,int loop, int (*callback)())
{
    vo_instance_t * output;
    uint32_t accel;

    accel =0;
	in_file = NULL;
	in_file = fopen (filename, "rb");
	if (in_file == NULL) {
		printf ( "Can not open mpeg file\n");
		return VF4DC_NOT_OPEN;
    }
    vo_accel (accel);
    output = vo_open (vo_drivers()[0].open);
    if (output == NULL) {
	printf ("Can not open output\n");
	return VF4DC_ERROR;
    }
    mpeg2_init (&mpeg2dec, accel, output);

    while(loop)
    {
	   ps_loop (callback);
	   if(stop)
		   break;
	   fseek(in_file,0,SEEK_SET);
	}
	fclose(in_file);
    mpeg2_close (&mpeg2dec);
    vo_close (output);
     return 0;
}
